<?php

/*********************************************************************************
 * TES is a Time and Expense Management program developed by
 * Initechs, LLC. Copyright (C) 2009 - 2010 Initechs LLC.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY INITECHS, INITECHS DISCLAIMS THE WARRANTY
 * OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, see http://www.gnu.org/licenses or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 *
 * You can contact Initechs headquarters at 1841 Piedmont Road, Suite 301,
 * Marietta, GA, USA. or at email address contact@initechs.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display od the "Initechs" logo.
 * If the display of the logo is not reasonably feasible for technical reasons,
 * the Appropriate Legal Notices must display the words "Powered by Initechs".

 ********************************************************************************/


$basedir = dirname(__FILE__) . '/..';

require_once("$basedir/initialize.php");
require_once("$basedir/accounts/DbObj.php");
require_once("$basedir/baseclass/DataValidator.php");

class EditProcess extends DataValidator
{

	protected function isValidPostData()
	{
		$ret_val = true;
		$dateFormat = getUserDateFormat();
			
		if (!$this->chkMandatory('name'))  $ret_val = false;
		if (!$this->chkMandatory('address1')) $ret_val = false;
		if (!$this->chkMandatory('city')) $ret_val = false;
		if (!$this->chkMandatory('billcycle')) $ret_val = false;
		if (!$this->chkMandatory('state')) $ret_val = false;
		if (!$this->chkMandatory('status')) $ret_val = false;
		if (!$this->chkMandatory('lastbilldate')) $ret_val = false;
		if (!isValidDate(trim($_POST['lastbilldate']), $dateFormat)) $ret_val = false;

		return $ret_val;
	}


	protected function processIncomingFormData()
	{
		$dateFormat = getUserDateFormat();
		try
		{
			$uid = trim($_POST['uid']);
			$aname = trim($_POST['name']);
			$ad1 = trim($_POST['address1']);
			$ad2 = trim($_POST['address2']);
			$acity = trim($_POST['city']);
			$astate = trim($_POST['state']);
			$apostal = trim($_POST['postalcode']);
			$acontact = trim($_POST['contact']);
			$aemail = trim($_POST['email']);
			$astat = trim($_POST['status']);
			$abillcycle = trim($_POST['billcycle']);
			$alastbilldate = convertdate(trim($_POST['lastbilldate']), $dateFormat, 'ymd');

			$Row = new AccountData();
			$Row->updateRow($uid, $aname, $ad1, $ad2, $acity, $astate, $apostal, $acontact, $aemail, $astat, $abillcycle, $alastbilldate);
			return true;
		}

		catch (iBLError $e)
		{
			$this->issueError("$e->errField", "$e->msgid", "$e->msgdta");
		}

		catch (iIDInUseException $e)
		{
			$this->issueError('accounts_id', 'er0004');
		}
		catch (Exception $e)
		{
			$this->issueError('nocategory', "er9998", "$e");
		}
	}

}

$goOnError = "../index.php?module={$_SESSION['module']}&action=Edit&uid={$_POST['uid']}&load=No";
$goOnSuccess = "../index.php?module={$_SESSION['module']}&action=Browse&uid={$_POST['uid']}";

$page = new EditProcess();
$page->processRequest();


?>